<?php

namespace Drupal\standalone\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\DependencyInjection\ContainerAwareTrait;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Page controller class for standalone content.
 */
class StandaloneContent extends ControllerBase implements ContainerAwareInterface {

  use ContainerAwareTrait;

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * StandaloneContent constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('entity_type.manager'));
  }

  /**
   * Callback for standalone page.
   *
   * @param string $type
   *   The entity type to render.
   * @param string $id
   *   The entity ID to render.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The markup for the page.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function page($type, $id) {
    // Check that the entity exists.
    $entity = $this->loadEntity($type, $id);
    if (empty($entity)) {
      throw new NotFoundHttpException();
    }

    // Check access to entity.
    if (!$entity->access('view')) {
      throw new AccessDeniedHttpException();
    }

    // Prepare the render array.
    $viewBuilder = $this->entityTypeManager->getViewBuilder($type);
    $viewMode = $this->getViewMode($type);
    $build = $viewBuilder->view($entity, $viewMode);

    // TODO: views.
    // TODO: Allow other modules to alter the output.
    // TODO: Perform end-of-request tasks in case there's any session activity.
    // TODO: dependency injection for rendering.
    return new Response('<html><body>' . render($build) . '</body></html>');
  }

  /**
   * Load the relevant entity.
   *
   * @param string $type
   *   The entity type to render.
   * @param string $id
   *   The entity ID to render.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The entity, if it exists.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function loadEntity($type, $id) {
    // Try loading the entity.
    $storage = $this->entityTypeManager->getStorage($type);
    $entity = $storage->load($id);
    if ($entity) {
      // We have the entity, so return it.
      return $entity;
    }

    return NULL;
  }

  /**
   * Get the view mode to use.
   *
   * @param string $type
   *   The entity type.
   *
   * @return string
   *   The view mode to use for rendering.
   */
  private function getViewMode($type) {
    $viewMode = '';
    switch ($type) {
      case 'node':
        $viewMode = 'default';
        break;

      case 'block':
        $viewMode = 'full';
        break;

    }
    return $viewMode;
  }

}
